@@ -2,8 +2,8 @@ library(gnm)
 set.seed(1)
 data(yaish)
 
-unidiff <- gnm(Freq ~ educ:orig + educ:dest +
+unidiff <- gnm(Freq ~ educ*orig + educ*dest +
                      Mult(Exp(-1 + educ), orig:dest), family = poisson,
-                     data = yaish)
+                     data = yaish, subset = (dest != 7))
 unidiff
 getContrasts(unidiff, grep("Mult.*educ", names(coef(unidiff))))
